@@ -91,7 +91,7 @@ void log_remove_model(seaudit_log_t * log, seaudit_model_t * model);
  * @return Vector of seaudit_message_t pointers.  Do not free() or
  * otherwise modify this vector or its contents.
  */
-apol_vector_t *log_get_messages(seaudit_log_t * log);
+const apol_vector_t *log_get_messages(const seaudit_log_t * log);
 
 /**
  * Get a vector of all malformed messages from this seaudit log
@@ -104,7 +104,7 @@ apol_vector_t *log_get_messages(seaudit_log_t * log);
  * @return Vector of strings.  Do not free() or otherwise modify this
  * vector or its contents.
  */
-apol_vector_t *log_get_malformed_messages(seaudit_log_t * log);
+const apol_vector_t *log_get_malformed_messages(const seaudit_log_t * log);
 
 /*************** messages (defined in message.c) ***************/
 
@@ -124,7 +124,7 @@ struct seaudit_message
 	union
 	{
 		seaudit_avc_message_t *avc;
-		seaudit_bool_message_t *bool;
+		seaudit_bool_message_t *boolm;
 		seaudit_load_message_t *load;
 	} data;
 };
@@ -269,7 +269,7 @@ void avc_message_free(seaudit_avc_message_t * avc);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *avc_message_to_string(seaudit_message_t * msg, const char *date);
+char *avc_message_to_string(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given an avc change message, allocate and return a string,
@@ -282,7 +282,7 @@ char *avc_message_to_string(seaudit_message_t * msg, const char *date);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *avc_message_to_string_html(seaudit_message_t * msg, const char *date);
+char *avc_message_to_string_html(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given an avc change message, allocate and return a string that
@@ -294,14 +294,14 @@ char *avc_message_to_string_html(seaudit_message_t * msg, const char *date);
  * error.  The caller is responsible for free()ing the string
  * afterwards.
  */
-char *avc_message_to_misc_string(seaudit_avc_message_t * avc);
+char *avc_message_to_misc_string(const seaudit_avc_message_t * avc);
 
 /*************** bool messages (defined in bool_message.c) ***************/
 
 typedef struct seaudit_bool_message_change
 {
 	/** pointer into log's bools BST */
-	char *bool;
+	char *boolean;
 	/** new value for the boolean */
 	int value;
 } seaudit_bool_message_change_t;
@@ -332,14 +332,14 @@ seaudit_bool_message_t *bool_message_create(void);
  *
  * @return 0 on success, < 0 on error.
  */
-int bool_change_append(seaudit_log_t * log, seaudit_bool_message_t * bool, char *name, int value);
+int bool_change_append(seaudit_log_t * log, seaudit_bool_message_t * boolm, const char *name, int value);
 
 /**
  * Deallocate all space associated with a boolean change message.
  *
  * @param msg If not NULL, message to free.
  */
-void bool_message_free(seaudit_bool_message_t * bool);
+void bool_message_free(seaudit_bool_message_t * boolm);
 
 /**
  * Given a boolean change message, allocate and return a string that
@@ -351,7 +351,7 @@ void bool_message_free(seaudit_bool_message_t * bool);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *bool_message_to_string(seaudit_message_t * msg, const char *date);
+char *bool_message_to_string(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given a boolean change message, allocate and return a string,
@@ -364,7 +364,7 @@ char *bool_message_to_string(seaudit_message_t * msg, const char *date);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *bool_message_to_string_html(seaudit_message_t * msg, const char *date);
+char *bool_message_to_string_html(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given a boolean change message, allocate and return a string that
@@ -377,7 +377,7 @@ char *bool_message_to_string_html(seaudit_message_t * msg, const char *date);
  * error.  The caller is responsible for free()ing the string
  * afterwards.
  */
-char *bool_message_to_misc_string(seaudit_bool_message_t * bool);
+char *bool_message_to_misc_string(const seaudit_bool_message_t * boolm);
 
 /*************** load messages (defined in load_message.c) ***************/
 
@@ -417,7 +417,7 @@ void load_message_free(seaudit_load_message_t * msg);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *load_message_to_string(seaudit_message_t * msg, const char *date);
+char *load_message_to_string(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given a load message, allocate and return a string, formatted in
@@ -430,7 +430,7 @@ char *load_message_to_string(seaudit_message_t * msg, const char *date);
  * @return String representation for message, or NULL upon error.  The
  * caller is responsible for free()ing the string afterwards.
  */
-char *load_message_to_string_html(seaudit_message_t * msg, const char *date);
+char *load_message_to_string_html(const seaudit_message_t * msg, const char *date);
 
 /**
  * Given a load message, allocate and return a string that gives
@@ -442,7 +442,7 @@ char *load_message_to_string_html(seaudit_message_t * msg, const char *date);
  * error.  The caller is responsible for free()ing the string
  * afterwards.
  */
-char *load_message_to_misc_string(seaudit_load_message_t * load);
+char *load_message_to_misc_string(const seaudit_load_message_t * load);
 
 /*************** model functions (defined in model.h) ***************/
 
@@ -474,6 +474,18 @@ void model_notify_filter_changed(seaudit_model_t * model, seaudit_filter_t * fil
 
 /*************** filter functions (defined in filter.c) ***************/
 
+/**
+ * Link a model to a filter.  Whenever the filter changes, it should
+ * call model_notify_filter_changed(); that way the model will
+ * recalculate itself.
+ *
+ * @param filter Filter to be watched.
+ * @param model Model that is watching.
+ */
+void filter_set_model(seaudit_filter_t * filter, seaudit_model_t * model);
+
+/********** more filter functions (defined in filter-internal.c) **********/
+
 typedef int (filter_read_func) (seaudit_filter_t * filter, const xmlChar * ch);
 
 struct filter_parse_state
@@ -491,7 +503,7 @@ struct filter_parse_state
 
     /****
         The following are to be considered private data and may only
-        be used by filter.c.
+        be used by filter-internal.c.
     ****/
     /** the most recently read string that was not part of a tag */
 	xmlChar *cur_string;
@@ -502,16 +514,6 @@ struct filter_parse_state
 	filter_read_func *cur_filter_read;
 };
 
-/**
- * Link a model to a filter.  Whenever the filter changes, it should
- * call model_notify_filter_changed(); that way the model will
- * recalculate itself.
- *
- * @param filter Filter to be watched.
- * @param model Model that is watching.
- */
-void filter_set_model(seaudit_filter_t * filter, seaudit_model_t * model);
-
 /**
  * Given a filter and a message, return non-zero if the msg is
  * accepted by the filter according to the filter's criteria.  If the
@@ -523,7 +525,7 @@ void filter_set_model(seaudit_filter_t * filter, seaudit_model_t * model);
  *
  * @return Non-zero if message is accepted, 0 if not.
  */
-int filter_is_accepted(seaudit_filter_t * filter, const seaudit_message_t * msg);
+int filter_is_accepted(const seaudit_filter_t * filter, const seaudit_message_t * msg);
 
 /**
  * Parse the given XML file and fill in the passed in struct.  The
@@ -546,7 +548,7 @@ int filter_parse_xml(struct filter_parse_state *state, const char *filename);
  *
  * @see seaudit_filter_create_from_file()
  */
-void filter_append_to_file(seaudit_filter_t * filter, FILE * file, int tabs);
+void filter_append_to_file(const seaudit_filter_t * filter, FILE * file, int tabs);
 
 /*************** sort functions (defined in sort.c) ***************/
 
@@ -586,7 +588,7 @@ seaudit_sort_t *sort_create_from_name(const char *name, int direction);
  *
  * @return Non-zero if sort supports the message, 0 if not.
  */
-int sort_is_supported(seaudit_sort_t * sort, const seaudit_message_t * msg);
+int sort_is_supported(const seaudit_sort_t * sort, const seaudit_message_t * msg);
 
 /**
  * Invoke a sort object's comparison function.
@@ -598,7 +600,7 @@ int sort_is_supported(seaudit_sort_t * sort, const seaudit_message_t * msg);
  * @return 0 if the messages are equivalent, < 0 if a is first, > 0 if
  * b is first.
  */
-int sort_comp(seaudit_sort_t * sort, const seaudit_message_t * a, const seaudit_message_t * b);
+int sort_comp(const seaudit_sort_t * sort, const seaudit_message_t * a, const seaudit_message_t * b);
 
 /**
  * Return the type of sort this sort object is.  The name is valid for
@@ -608,7 +610,7 @@ int sort_comp(seaudit_sort_t * sort, const seaudit_message_t * a, const seaudit_
  *
  * @return Type of sort this object is.
  */
-const char *sort_get_name(seaudit_sort_t * sort);
+const char *sort_get_name(const seaudit_sort_t * sort);
 
 /**
  * Return the sort direction for a sort object.
@@ -617,7 +619,7 @@ const char *sort_get_name(seaudit_sort_t * sort);
  *
  * @return Non-negative for ascending, negative for descending.
  */
-int sort_get_direction(seaudit_sort_t * sort);
+int sort_get_direction(const seaudit_sort_t * sort);
 
 /*************** error handling code (defined in log.c) ***************/
 
@@ -636,10 +638,10 @@ int sort_get_direction(seaudit_sort_t * sort);
  * SEAUDIT_MSG_WARN, or SEAUDIT_MSG_INFO.
  * @param fmt Format string to print, using syntax of printf(3).
  */
-extern void seaudit_handle_msg(seaudit_log_t * log, int level, const char *fmt, ...);
+extern void seaudit_handle_msg(const seaudit_log_t * log, int level, const char *fmt, ...);
 
 __attribute__ ((format(printf, 3, 4)))
-extern void seaudit_handle_msg(seaudit_log_t * log, int level, const char *fmt, ...);
+extern void seaudit_handle_msg(const seaudit_log_t * log, int level, const char *fmt, ...);
 
 #undef ERR
 #undef WARN
